﻿namespace UltraGridApplication
{
    using Infragistics.Win.UltraWinGrid;
    using System;
    using System.Data;
    using System.Drawing;
    using System.Linq;
    using System.Threading;
    using System.Windows.Forms;

    public partial class Form1 : Form
    {
        private delegate void SyncWidthsEventHandler(UltraGridColumn column, Control control);

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.ultraGrid1.DataSource = this.GetData();
            this.ultraButton1.Name = "Id";
            this.ultraButton2.Name = "Name";
            this.ultraButton3.Name = "Quantity";
            TimerCallback handler = new TimerCallback(InitialWidthSync);
            var timer = new System.Threading.Timer(handler, null, 1, -1);    
            ultraGrid1.PropertyChanged += ultraGrid1_PropertyChanged;
        }

        public void InitialWidthSync(object state)
        {
            SyncWidthsEventHandler handler = new SyncWidthsEventHandler(SyncWidths);
            var gridColumns = this.ultraGrid1.DisplayLayout.Bands[0].Columns;

            foreach (var column in gridColumns)
            {
                Control controlToResize =
                    this.Controls.OfType<Control>()
                    .Where(control => control.Name == column.Header.Caption)
                    .FirstOrDefault();
                this.Invoke(handler, new object[] {column, controlToResize});
            }
        }

        private void SyncWidths(UltraGridColumn column, Control control)
        {
            this.Controls[column.Key].Width = column.Width;
            if (column.Header.GetUIElement() != null)
            {
                Point headerLocation = column.Header.GetUIElement().Rect.Location;
                headerLocation = this.PointToClient(ultraGrid1.PointToScreen(headerLocation));
                this.Controls[column.Key].Left = headerLocation.X;
            }
        }

        void ultraGrid1_PropertyChanged(object sender, Infragistics.Win.PropertyChangedEventArgs e)
        {
            if (e.ChangeInfo.FindPropId(Infragistics.Win.UltraWinGrid.PropertyIds.Width) != null)
            {
                if (e.ChangeInfo.FindPropId(Infragistics.Win.UltraWinGrid.PropertyIds.Header) != null)
                {
                     //REPOSITION CONTROLS USING WIDTHS OR VISIBLE COLUMNS TO CALCULATE LOCATION
                    var header = e.ChangeInfo.FindTrigger(null);
                    var column = header.Source as UltraGridColumn;
                    if (column != null)
                    {
                        this.SyncWidths(column, this.Controls[column.Key]);
                    }
                }
            }
        }

        private DataSet GetData()
        {
            DataSet dataSet = new DataSet();
            DataTable table = dataSet.Tables.Add("table1");
            table.Columns.Add("Id", typeof(int)).AutoIncrement = true;
            table.Columns.Add("Name", typeof(string));
            table.Columns.Add("Quantity", typeof(int));
            dataSet = AddData(dataSet);
            return dataSet;
        }

        private DataSet AddData(DataSet dataSet)
        {
            dataSet.Tables["table1"].Rows.Add(new object[] { null, "Beverage", 50 });
            dataSet.Tables["table1"].Rows.Add(new object[] { null, "Appliances", 5, });
            return dataSet;
        }

        private void ultraGrid1_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            e.Layout.AutoFitStyle = Infragistics.Win.UltraWinGrid.AutoFitStyle.ResizeAllColumns;           
        }
    }
}
